package de.hsrm.swt.vokabeltrainer.lax.data;

/**
 * Datenobjekt fr eine Vokabel: enthlt folgende Felder:<br/>
 * <ul>
 *   <li>Frage (in Quellsprache)</li> 
 *   <li>Antwort (in Zielsprache)</li>
 * </ul><br/>
 * Beides sind zwar bei der Speicherung der Daten (oder bei Verwendung z.B. in der @see Vokabelpruefung ) 
 * Pflichtfelder, aber whrend des sonstigen Programmlaufs sind sie auch auf NULL oder Leerstring setzbar.  
 * 
 * @author Wolfgang Knauf
 * 
 */
public class Vokabel
{
  // Reprsentationsinvariante:
  // sFrage und sAntwort drfen NULL sein, drfen aber nicht in diesem Zustand gespeichert werden.
  
  /**
   * Die Frage
   */
  private String sFrage;

  /**
   * Die Antwort
   */
  private String sAntwort;

  /**
   * Konstruktor ohne Parameter. Ist in dieser Variante erlaubt, da 
   * Frage und Antwort leer sein drfen
   * @post Frage == null && Antwort == null
   * 
   */
  public Vokabel()
  { 
  }
  
  /**
   * Konstruktor, dem Frage und Antwort bergeben werden.
   * 
   * @param sFrage
   *          Frage (auch null oder leer erlaubt)
   * @param sAntwort
   *          Antwort (auch null oder leer erlaubt)
   * @post Frage = bergebene Frage, Antwort = bergebene Antwort
   */
  public Vokabel(String sFrage, String sAntwort)
  {
    this.sFrage = sFrage;
    this.sAntwort = sAntwort;
  }

  /**
   * Abrufen der Frage.
   * 
   * @return Frage
   */
  public String getFrage()
  {
    return sFrage;
  }

  /**
   * Setzen der Frage.
   * 
   * @param sFrage
   *          Neue Frage (auch null oder leer erlaubt)
   * @post Frage = bergebene Frage
   */
  public void setFrage(String sFrage)
  {
    this.sFrage = sFrage;
  }

  /**
   * Abrufen der Antwort.
   * 
   * @return Antwort
   */
  public String getAntwort()
  {
    return sAntwort;
  }

  /**
   * Setzen der Antwort.
   * 
   * @param sAntwort
   *          Neue Antwort (auch null oder leer erlaubt)
   * @post Antwort = bergebene Antwort
   */
  public void setAntwort(String sAntwort)
  {
    this.sAntwort = sAntwort;
  }


  /**
   * Gibt die gesamte interne Reprsentation der Klasse (Abstraktionsfunktion)
   * 
   * @return Interne Reprsentation der Klasse fr Debugging-Zwecke mit allen
   *         Informationen
   */
  @Override
  public String toString()
  {
    return "Frage: " + this.sFrage + ", Antwort: " + this.sAntwort;

  }
}
