package de.hsrm.swt.vokabeltrainer.strict.data;

/**
 * Datenobjekt fr eine Vokabel:  enthlt folgende Felder:<br/>
 * <ul>
 *   <li>Frage (in Quellsprache)</li>
 *   <li>Antwort (in Zielsprache)</li>
 * </ul>
 * <br/>
 * Beides sind Pflichtfelder und werden deshalb per Konstruktor bergeben.<br/>
 * 
 * @author Wolfgang Knauf
 * 
 */
public class Vokabel
{
  // Reprsentationsinvariante:
  // sFrage != null und sFrage != "" und sAntwort != null && sAntwort != ""
  
  /**
   * Die Frage, nicht leer.
   */
  private String sFrage;

  /**
   * Die Antwort, nicht leer.
   */
  private String sAntwort;

  /**
   * Konstruktor, dem Frage und Antwort bergeben werden.
   * 
   * @param sFrage
   *          Frage, nicht null oder leer
   * @param sAntwort
   *          Antwort, nicht null oder leer
   * @throws IllegalArgumentException
   *           Falls Frage oder Antwort null/leer sind.
   * @post Frage = bergebene Frage, Antwort = bergebene Antwort
   */
  public Vokabel(String sFrage, String sAntwort)
  {
    if (sFrage == null)
    {
      throw new IllegalArgumentException("Frage darf nicht null sein");
    }
    else if (sFrage.isEmpty())
    {
      throw new IllegalArgumentException("Frage darf nicht leer sein");
    }

    if (sAntwort == null)
    {
      throw new IllegalArgumentException("Antwort darf nicht null sein");
    }
    else if (sAntwort.isEmpty())
    {
      throw new IllegalArgumentException("Antwort darf nicht leer sein");
    }

    this.sFrage = sFrage;
    this.sAntwort = sAntwort;

    // Check der Reprsentationsinvariante:
    this.checkRep();
  }

  /**
   * Abrufen der Frage.
   * 
   * @return Frage
   */
  public String getFrage()
  {
    // Check der Reprsentationsinvariante:
    this.checkRep();

    return sFrage;
  }

  /**
   * Setzen der Frage.
   * 
   * @param sFrage
   *          Neue Frage, darf nicht leer sein.
   * @exception IllegalArgumentException
   *              Falls Parameter "sFrage" leer oder NULL ist.
   * @post Frage = bergebene Frage
   */
  public void setFrage(String sFrage)
  {
    if (sFrage == null)
    {
      throw new IllegalArgumentException("Frage darf nicht null sein");
    }
    else if (sFrage.isEmpty())
    {
      throw new IllegalArgumentException("Frage darf nicht leer sein");
    }

    this.sFrage = sFrage;

    // Check der Reprsentationsinvariante:
    this.checkRep();
  }

  /**
   * Abrufen der Antwort.
   * 
   * @return Antwort
   */
  public String getAntwort()
  {
    // Check der Reprsentationsinvariante:
    this.checkRep();

    return sAntwort;
  }

  /**
   * Setzen der Antwort.
   * 
   * @param sAntwort
   *          Neue Antwort, darf nicht leer sein.
   * @exception IllegalArgumentException
   *              Falls Parameter "sAntwort" leer oder NULL ist.
   * @post Antwort = bergebene Antwort
   */
  public void setAntwort(String sAntwort)
  {
    if (sAntwort == null)
    {
      throw new IllegalArgumentException("Antwort darf nicht null sein");
    }
    else if (sAntwort.isEmpty())
    {
      throw new IllegalArgumentException("Antwort darf nicht leer sein");
    }

    this.sAntwort = sAntwort;

    // Check der Reprsentationsinvariante:
    this.checkRep();
  }

  /**
   * Prft die Reprsentationsinvariante - siehe Klassenkommentar. Wirft
   * Exceptions, wenn irgendetwas bse ist. Ansonsten macht die Methode nichts.
   */
  private void checkRep()
  {
    if (this.sFrage == null)
    {
      throw new IllegalStateException("Frage darf nicht null sein");
    }
    else if (this.sFrage.isEmpty())
    {
      throw new IllegalStateException("Frage darf nicht leer sein");
    }

    if (this.sAntwort == null)
    {
      throw new IllegalStateException("Antwort darf nicht null sein");
    }
    else if (this.sAntwort.isEmpty())
    {
      throw new IllegalStateException("Antwort darf nicht leer sein");
    }
  }

  /**
   * Gibt die gesamte interne Reprsentation der Klasse (Abstraktionsfunktion)
   * 
   * @return Interne Reprsentation der Klasse fr Debugging-Zwecke mit allen
   *         Informationen
   */
  @Override
  public String toString()
  {
    return "Frage: " + this.sFrage + ", Antwort: " + this.sAntwort;

  }
}
