package de.hsrm.swt.vokabeltrainer.strict.test;

import java.util.Vector;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import de.hsrm.swt.vokabeltrainer.lax.data.Vokabel;
import de.hsrm.swt.vokabeltrainer.lax.logik.Vokabelpruefung;

/**Unit-Tests fr die Klasse {@link Vokabelpruefung}
 * 
 * @author Wolfgang Knauf
 *
 */
public class VokabelpruefungTest
{

  /**Liste der zu prfenden Vokabeln wird in "{@link VokabelpruefungTest.init}" erzeugt
   * und enthlt hier immer zwei Vokabeln.
   */
  private Vector<Vokabel> listVokabeln;

  /**Test "zweimal korrekte Antwort eingeben".
   * 
   * Testfall 001
   */
  @Test
  public void testVokabelpruefungAllesKorrekt()
  {

    Vokabelpruefung pruefung = new Vokabelpruefung(listVokabeln);

    Vokabel v = pruefung.start();
    Assert.assertEquals("hallo", v.getFrage());

    v = pruefung.checkVokabel("hello");
    Assert.assertEquals("welt", v.getFrage());
    Assert.assertEquals(1, pruefung.getStatistikKorrekt());
    Assert.assertEquals(0, pruefung.getStatistikFalsch());
    Assert.assertEquals(0, pruefung.getAktuelleVokabelFalsch());

    v = pruefung.checkVokabel("world");
    Assert.assertNull(v);
    Assert.assertEquals(2, pruefung.getStatistikKorrekt());
    Assert.assertEquals(0, pruefung.getStatistikFalsch());
    Assert.assertEquals(0, pruefung.getAktuelleVokabelFalsch());
  }
  
  /**Test "einmal korrekte, dann dreimal falsche Antwort eingeben".
   * 
   * Testfall 002
   */
  @Test
  public void testVokabelpruefungKorrektUndFalsch()
  {

    Vokabelpruefung pruefung = new Vokabelpruefung(listVokabeln);

    Vokabel v = pruefung.start();
    Assert.assertEquals("hallo", v.getFrage());

    v = pruefung.checkVokabel("hello");
    Assert.assertEquals("welt", v.getFrage());
    Assert.assertEquals(1, pruefung.getStatistikKorrekt());
    Assert.assertEquals(0, pruefung.getStatistikFalsch());
    Assert.assertEquals(0, pruefung.getAktuelleVokabelFalsch());

    v = pruefung.checkVokabel("wurld");
    Assert.assertEquals("welt", v.getFrage());
    Assert.assertEquals(1, pruefung.getStatistikKorrekt());
    Assert.assertEquals(0, pruefung.getStatistikFalsch());
    Assert.assertEquals(1, pruefung.getAktuelleVokabelFalsch());

    v = pruefung.checkVokabel("warld");
    Assert.assertEquals("welt", v.getFrage());
    Assert.assertEquals(1, pruefung.getStatistikKorrekt());
    Assert.assertEquals(0, pruefung.getStatistikFalsch());
    Assert.assertEquals(2, pruefung.getAktuelleVokabelFalsch());

    v = pruefung.checkVokabel("wirld");
    Assert.assertNull(v);
    Assert.assertEquals(1, pruefung.getStatistikKorrekt());
    Assert.assertEquals(1, pruefung.getStatistikFalsch());
    //"aktuell falsch" wird bei dreimal falsch wieder auf 0 gesetzt.
    Assert.assertEquals(0, pruefung.getAktuelleVokabelFalsch());

  }

  /**Test "erste Vokabel korrekt beantworten, dann zweite Vokabel
   * zweimal falsch und dann kortek beantworten".
   * 
   * Hinweis: dieser Test ist unntig, weil die anderen beiden
   * Tests schon den kompletten Code von {@link Vokabelpruefung#checkVokabel}
   * abdecken.
   * 
   * Testfall 003
   */
  @Test
  public void testVokabelpruefungKorrektUndFalschKorrekt()
  {

    Vokabelpruefung pruefung = new Vokabelpruefung(listVokabeln);

    Vokabel v = pruefung.start();
    Assert.assertEquals("hallo", v.getFrage());

    v = pruefung.checkVokabel("hello");
    Assert.assertEquals("welt", v.getFrage());
    Assert.assertEquals(1, pruefung.getStatistikKorrekt());
    Assert.assertEquals(0, pruefung.getStatistikFalsch());
    Assert.assertEquals(0, pruefung.getAktuelleVokabelFalsch());

    v = pruefung.checkVokabel("wurld");
    Assert.assertEquals("welt", v.getFrage());
    Assert.assertEquals(1, pruefung.getStatistikKorrekt());
    Assert.assertEquals(0, pruefung.getStatistikFalsch());
    Assert.assertEquals(1, pruefung.getAktuelleVokabelFalsch());

    v = pruefung.checkVokabel("warld");
    Assert.assertEquals("welt", v.getFrage());
    Assert.assertEquals(1, pruefung.getStatistikKorrekt());
    Assert.assertEquals(0, pruefung.getStatistikFalsch());
    Assert.assertEquals(2, pruefung.getAktuelleVokabelFalsch());

    v = pruefung.checkVokabel("world");
    Assert.assertNull(v);

    Assert.assertEquals(2, pruefung.getStatistikKorrekt());
    Assert.assertEquals(0, pruefung.getStatistikFalsch());
    Assert.assertEquals(0, pruefung.getAktuelleVokabelFalsch());

  }

  /**Initialisierung des Tests: erzeugen der Liste der zu prfenden Vokabeln
   * (zwei Stck).
   * 
   * Hinweis: sinnvoller wre hier, das als "BeforeClass"-Methode zu deklarieren,
   * da die Initialisierung nur einmalig fr alle Tests erfolgen muss.
   * 
   */
  @Before
  public void init()
  {
    this.listVokabeln = new Vector<Vokabel>();

    Vokabel v = new Vokabel("hallo", "hello");
    listVokabeln.add(v);

    v = new Vokabel("welt", "world");
    listVokabeln.add(v);
  }

  
}
